==> org-lsst-ccs-messaging-layer-2.0.0-sources.jar: org/lsst/ccs/messaging/MessagingLayer.java <==
package org.lsst.ccs.messaging;

import java.io.IOException;
import org.lsst.ccs.bus.definition.Bus;
import org.lsst.ccs.bus.messages.BusMessage;

/**
 * Interface providing methods to connect/disconnect MessagingAccessLayer objects to/from the CCS buses and to
 * send messages over the buses.
 * An instance of a MessagingLayer is created via the MessagingManager.
 * When connecting a MessagingAccessLayer, its name is used to uniquely identify it on the buses.
 * If such name is not unique, a DuplicateAgentNameException is thrown.
 * 
 * @author emarin
 */
public interface MessagingLayer {
    
    /**
     * Sends a message on a bus for a given MessagingAccessLayer.
     * The name of the MessagingAccessLayer determines the origin of the bus message.
     * note : that BusMessage should have sender and
     * destination information but it is not the role of the communication layer
     * to parse destination information such as "subsystem1, subsystem2" or
     * "subsystem3/module"a.
     * @param <T> the type of message sent on the bus
     * @param senderAgent provides the name as the origin of the message 
     * @param bus the bus on which to send the message
     * @param msg the message
     * @param destination could be empty (means broadcast) of a single "" or a single"*" (again means broadcast)
     * or a list of destinations (DO NOT use things such as "*", "dest1", "dest2" : this will not work!)
     * @throws IOException this could have a list of causes if some destinations fail.
     * as much as possible implementers will ensure that all correct destinationa are adressed:
     * if some fail it is not mandatory to report with a special subclass of IOexception that lists all destination
     * that failed.
     * @see org.lsst.ccs.bus.DestinationsException
     */
    <T extends BusMessage> void sendMessage (String senderAgent, Bus<T> bus, T msg, String... destination) throws IOException;
    
    /**
     * Connects a MessagingAccessLayer object on the buses by using its name as a 
     * unique identifier.
     * Upon creation, if needed, all needed buses are created.
     * Depending on the underlying implementation, the MessagingLayer allows this method
     * to be called more than once, or throws a RuntimeException.
     * TBD : should all the buses be created or just the ones specified by the MessagingAccessLayer ?
     * @param layer The layer to be connected on the buses
     * @throws DuplicateAgentNameException if the name of MessagingAccessLayer is not unique on the buses
     */
    void connect(MessagingAccessLayer accessLayer) throws DuplicateAgentNameException, IOException;
    
    /**
     * Remove the entry point to the buses for the given MessagingAccessLayer.
     * At this point, the CCS buses might be closed.
     * 
     * @param layer 
     */
    void disconnect(MessagingAccessLayer accessLayer);
}

==> org-lsst-ccs-messaging-layer-2.0.2-sources.jar: org/lsst/ccs/messaging/MessagingLayer.java <==
package org.lsst.ccs.messaging;

import java.io.IOException;
import org.lsst.ccs.bus.definition.Bus;
import org.lsst.ccs.bus.messages.BusMessage;

/**
 * Interface providing methods to connect/disconnect MessagingAccessLayer objects to/from the CCS buses and to
 * send messages over the buses.
 * An instance of a MessagingLayer is created via the MessagingManager.
 * When connecting a MessagingAccessLayer, its name is used to uniquely identify it on the buses.
 * If such name is not unique, a DuplicateAgentNameException is thrown.
 * 
 * @author emarin
 */
public interface MessagingLayer {
    
    /**
     * Sends a message on a bus for a given MessagingAccessLayer.
     * The name of the MessagingAccessLayer determines the origin of the bus message.
     * note : that BusMessage should have sender and
     * destination information but it is not the role of the communication layer
     * to parse destination information such as "subsystem1, subsystem2" or
     * "subsystem3/module"a.
     * @param <T> the type of message sent on the bus
     * @param senderAgent provides the name as the origin of the message 
     * @param bus the bus on which to send the message
     * @param msg the message
     * @throws IOException this could have a list of causes if some destinations fail.
     * as much as possible implementers will ensure that all correct destinationa are adressed:
     * if some fail it is not mandatory to report with a special subclass of IOexception that lists all destination
     * that failed.
     * @see org.lsst.ccs.bus.DestinationsException
     */
    <T extends BusMessage> void sendMessage (String senderAgent, Bus<T> bus, T msg) throws IOException;
    
    /**
     * Connects a MessagingAccessLayer object on the buses by using its name as a 
     * unique identifier.
     * Upon creation, if needed, all needed buses are created.
     * Depending on the underlying implementation, the MessagingLayer allows this method
     * to be called more than once, or throws a RuntimeException.
     * TBD : should all the buses be created or just the ones specified by the MessagingAccessLayer ?
     * @param accessLayer The layer to be connected on the buses
     * @throws DuplicateAgentNameException if the name of MessagingAccessLayer is not unique on the buses
     */
    void connect(MessagingAccessLayer accessLayer) throws DuplicateAgentNameException, IOException;
    
    /**
     * Remove the entry point to the buses for the given MessagingAccessLayer.
     * At this point, the CCS buses might be closed.
     * @param accessLayer The layer to be connected on the buses
     */
    void disconnect(MessagingAccessLayer accessLayer);
}
